/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of the demonstration applications of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/
#include <QtCore/QBuffer>
#include <QtCore/QDir>
#include <QtCore/QLibraryInfo>
#include <QtCore/QSettings>
#include <QtCore/QTextStream>
#include <QtCore/QTranslator>
#include <QtCore/QDebug>
#include <QtGui/QDesktopServices>
#include <QtGui/QFileOpenEvent>
#include <QtGui/QMessageBox>
#include <QtNetwork/QLocalServer>
#include <QtNetwork/QLocalSocket>
#include <QtNetwork/QNetworkProxy>
#include <QtNetwork/QSslSocket>
#include <QtWebKit/QWebSettings>
#include <QFileDialog>


#include "browserapplication.h"
#include "bookmarks.h"
#include "browsermainwindow.h"
#include "cookiejar.h"
#include "downloadmanager.h"
#include "history.h"
#include "networkaccessmanager.h"
#include "tabwidget.h"
#include "webview.h"
#include "vmessagebox.h"
#include "serversettings.h"
#include "torsettings.h"
#include "net.h"
#include "file.h"
#include "html.h"
#include "stringutil.h"
#include "procutil.h"
#include "clientstatusevent.h"
#include "dangerousversionevent.h"
#include "dangerousportevent.h"
#include "controlpasswordinputdialog.h"
#include "settings.h"
#include "countryinfo.h"


/** Only allow 'New Identity' to be clicked once every 10 seconds. */
#define MIN_NEWIDENTITY_INTERVAL   (10*1000)

/* Startup progress milestones */
#define STARTUP_PROGRESS_STARTING          0
#define STARTUP_PROGRESS_CONNECTING       10
#define STARTUP_PROGRESS_AUTHENTICATING   20
#define STARTUP_PROGRESS_BOOTSTRAPPING    30
#define STARTUP_PROGRESS_CIRCUITBUILD     75
#define STARTUP_PROGRESS_MAXIMUM          (STARTUP_PROGRESS_BOOTSTRAPPING+100)

/* Static member variables */
TorControl* BrowserApplication::_torControl = 0;  /**< Main TorControl object.          */
Log BrowserApplication::_log;


DownloadManager *BrowserApplication::s_downloadManager = 0;
HistoryManager *BrowserApplication::s_historyManager = 0;
NetworkAccessManager *BrowserApplication::s_networkAccessManager = 0;
BookmarksManager *BrowserApplication::s_bookmarksManager = 0;

BrowserApplication::BrowserApplication(int &argc, char **argv)
    : QApplication(argc, argv)
    , m_localServer(0)
{
    qInstallMsgHandler(qt_msg_handler);	
    
     loadDefaultCaCertificates();
	
    QCoreApplication::setOrganizationName(QLatin1String("Agiag"));
    QCoreApplication::setApplicationName(QLatin1String("TorNavigator"));
    QCoreApplication::setApplicationVersion(QLatin1String("0.1"));
#ifdef Q_WS_QWS
    // Use a different server name for QWS so we can run an X11
    // browser and a QWS browser in parallel on the same machine for
    // debugging
    QString serverName = QCoreApplication::applicationName() + QLatin1String("_qws");
#else
    QString serverName = QCoreApplication::applicationName();
#endif
    QLocalSocket socket;
    socket.connectToServer(serverName);
    if (socket.waitForConnected(500)) {
        QTextStream stream(&socket);
        QStringList args = QCoreApplication::arguments();
        if (args.count() > 1)
            stream << args.last();
        else
            stream << QString();
        stream.flush();
        socket.waitForBytesWritten();
        return;
    }

#if defined(Q_WS_MAC)
    QApplication::setQuitOnLastWindowClosed(false);
#else
    QApplication::setQuitOnLastWindowClosed(true);
#endif

    m_localServer = new QLocalServer(this);
    connect(m_localServer, SIGNAL(newConnection()),
            this, SLOT(newLocalSocketConnection()));
    if (!m_localServer->listen(serverName)) {
        if (m_localServer->serverError() == QAbstractSocket::AddressInUseError
            && QFile::exists(m_localServer->serverName())) {
            QFile::remove(m_localServer->serverName());
            m_localServer->listen(serverName);
        }
    }

#ifndef QT_NO_OPENSSL
    if (!QSslSocket::supportsSsl()) {
    QMessageBox::information(0, "Demo Browser",
                 "This system does not support OpenSSL. SSL websites will not be available.");
    }
#endif

    QDesktopServices::setUrlHandler(QLatin1String("http"), this, "openUrl");
    
    QString localSysName = QLocale::system().name();
    installTranslator(QLatin1String("qt_") + localSysName); 
   
    
    QString appPath = QApplication::applicationDirPath ()+"/config.ini"; 
    QSettings settings(appPath,QSettings::IniFormat); 
    settings.beginGroup(QLatin1String("sessions"));
    m_lastSession = settings.value(QLatin1String("lastSession")).toByteArray();
    settings.endGroup();

#if defined(Q_WS_MAC)
    connect(this, SIGNAL(lastWindowClosed()),
            this, SLOT(lastWindowClosed()));
#endif

/* Create a new TorControl object, used to communicate with Tor */
  _torControl = new TorControl();
 
  connect(_torControl, SIGNAL(started()),this, SLOT(started()));
  connect(_torControl, SIGNAL(startFailed(QString)),this,   SLOT(startFailed(QString)));
  connect(_torControl, SIGNAL(stopped(int, QProcess::ExitStatus)),this,SLOT(stopped(int, QProcess::ExitStatus)));
  connect(_torControl, SIGNAL(connected()), this, SLOT(connected()));
  connect(_torControl, SIGNAL(disconnected()), this, SLOT(disconnected()));
  connect(_torControl, SIGNAL(connectFailed(QString)),this,   SLOT(connectFailed(QString)));
  connect(_torControl, SIGNAL(authenticated()), this, SLOT(authenticated()));
  connect(_torControl, SIGNAL(authenticationFailed(QString)),this,SLOT(authenticationFailed(QString)));
  _torControl->setEvent(TorEvents::ClientStatus,  this, true);
  _torControl->setEvent(TorEvents::GeneralStatus, this, true);
  
  /* Catch signals when the application is running or shutting down */
  connect(vApp, SIGNAL(running()), this, SLOT(mainLoopRunning()));
  connect(vApp, SIGNAL(aboutToQuit()), this, SLOT(aboutToQuit()));


  /* Start with Tor initially stopped */
  _status = Unset;
  _isTorNavigatorRunningTor = false;
  updateTorStatus(Stopped);
 

    QTimer::singleShot(0, this, SLOT(postLaunch()));

}

BrowserApplication::~BrowserApplication()
{
    delete s_downloadManager;
    for (int i = 0; i < m_mainWindows.size(); ++i) {
        BrowserMainWindow *window = m_mainWindows.at(i);
     if(window)delete window;
    }
    if (s_networkAccessManager) delete s_networkAccessManager;
    if (s_bookmarksManager) delete s_bookmarksManager;
    
    //_torControl->kill();
    m_PrivoxyController.privoxyProcess->kill();
}

#if defined(Q_WS_MAC)
void BrowserApplication::lastWindowClosed()
{
    clean();
    BrowserMainWindow *mw = new BrowserMainWindow;
    mw->slotHome();
    m_mainWindows.prepend(mw);
}
#endif

BrowserApplication *BrowserApplication::instance()
{
    return (static_cast<BrowserApplication *>(QCoreApplication::instance()));
}

#if defined(Q_WS_MAC)
#include <QtGui/QMessageBox>
void BrowserApplication::quitBrowser()
{
    clean();
    int tabCount = 0;
    for (int i = 0; i < m_mainWindows.count(); ++i) {
        tabCount =+ m_mainWindows.at(i)->tabWidget()->count();
    }

    if (tabCount > 1) {
        int ret = QMessageBox::warning(mainWindow(), QString(),
                           tr("There are %1 windows and %2 tabs open\n"
                              "Do you want to quit anyway?").arg(m_mainWindows.count()).arg(tabCount),
                           QMessageBox::Yes | QMessageBox::No,
                           QMessageBox::No);
        if (ret == QMessageBox::No)
            return;
    }

    exit(0);
}
#endif

/*!
    Any actions that can be delayed until the window is visible
 */
void BrowserApplication::postLaunch()
{
    QString directory = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
    if (directory.isEmpty())
        directory = QDir::homePath() + QLatin1String("/.") + QCoreApplication::applicationName();
    QWebSettings::setIconDatabasePath(directory);

    setWindowIcon(QIcon(QLatin1String(":browser.svg")));

    updatePrivoxyConfig();
    loadSettings(); 
    
     if(!_torControl->isConnected())
           	{
         	// let's start privoxy server  
            m_PrivoxyController.startPrivoxy();
            // start tor   
            startTor(); 
            //QMessageBox::warning(0,"test1","Start tor and privoxy"); 
        	} 

    // newMainWindow() needs to be called in main() for this to happen
    if (m_mainWindows.count() > 0) {
        QStringList args = QCoreApplication::arguments();
        if (args.count() > 1)
        {        	
         mainWindow()->loadPage(args.last());        	
       	}
        else
        {                       
         mainWindow()->slotHome();	
       	}  
    }
    BrowserApplication::historyManager();
  
 
}

void BrowserApplication::loadSettings()
{
    //QSettings settings;
    QString appPath = QApplication::applicationDirPath ()+"/config.ini"; 
    QSettings settings(appPath,QSettings::IniFormat); 
    settings.beginGroup(QLatin1String("websettings"));

    QWebSettings *defaultSettings = QWebSettings::globalSettings();
    QString standardFontFamily = defaultSettings->fontFamily(QWebSettings::StandardFont);
    int standardFontSize = defaultSettings->fontSize(QWebSettings::DefaultFontSize);
    QFont standardFont = QFont(standardFontFamily, standardFontSize);
    standardFont = qVariantValue<QFont>(settings.value(QLatin1String("standardFont"), standardFont));
    defaultSettings->setFontFamily(QWebSettings::StandardFont, standardFont.family());
    defaultSettings->setFontSize(QWebSettings::DefaultFontSize, standardFont.pointSize());

    QString fixedFontFamily = defaultSettings->fontFamily(QWebSettings::FixedFont);
    int fixedFontSize = defaultSettings->fontSize(QWebSettings::DefaultFixedFontSize);
    QFont fixedFont = QFont(fixedFontFamily, fixedFontSize);
    fixedFont = qVariantValue<QFont>(settings.value(QLatin1String("fixedFont"), fixedFont));
    defaultSettings->setFontFamily(QWebSettings::FixedFont, fixedFont.family());
    defaultSettings->setFontSize(QWebSettings::DefaultFixedFontSize, fixedFont.pointSize());

    defaultSettings->setAttribute(QWebSettings::JavascriptEnabled, settings.value(QLatin1String("enableJavascript"), true).toBool());
    defaultSettings->setAttribute(QWebSettings::PluginsEnabled, settings.value(QLatin1String("enablePlugins"), true).toBool());

    QUrl url = settings.value(QLatin1String("userStyleSheet")).toUrl();
    defaultSettings->setUserStyleSheetUrl(url);

    settings.endGroup();
}

QList<BrowserMainWindow*> BrowserApplication::mainWindows()
{
    clean();
    QList<BrowserMainWindow*> list;
    for (int i = 0; i < m_mainWindows.count(); ++i)
        list.append(m_mainWindows.at(i));
    return list;
}

void BrowserApplication::clean()
{
    // cleanup any deleted main windows first
    for (int i = m_mainWindows.count() - 1; i >= 0; --i)
        if (m_mainWindows.at(i).isNull())
            m_mainWindows.removeAt(i);
}

void BrowserApplication::saveSession()
{
    QWebSettings *globalSettings = QWebSettings::globalSettings();
    if (globalSettings->testAttribute(QWebSettings::PrivateBrowsingEnabled))
        return;

    clean();

    //QSettings settings;
    QString appPath = QApplication::applicationDirPath ()+"/config.ini"; 
    QSettings settings(appPath,QSettings::IniFormat); 
    settings.beginGroup(QLatin1String("sessions"));

    QByteArray data;
    QBuffer buffer(&data);
    QDataStream stream(&buffer);
    buffer.open(QIODevice::ReadWrite);

    stream << m_mainWindows.count();
    for (int i = 0; i < m_mainWindows.count(); ++i)
        stream << m_mainWindows.at(i)->saveState();
    settings.setValue(QLatin1String("lastSession"), data);
    settings.endGroup();
}

bool BrowserApplication::canRestoreSession() const
{
    return !m_lastSession.isEmpty();
}

void BrowserApplication::restoreLastSession()
{
    QList<QByteArray> windows;
    QBuffer buffer(&m_lastSession);
    QDataStream stream(&buffer);
    buffer.open(QIODevice::ReadOnly);
    int windowCount;
    stream >> windowCount;
    for (int i = 0; i < windowCount; ++i) {
        QByteArray windowState;
        stream >> windowState;
        windows.append(windowState);
    }
    for (int i = 0; i < windows.count(); ++i) {
        BrowserMainWindow *newWindow = 0;
        if (m_mainWindows.count() == 1
            && mainWindow()->tabWidget()->count() == 1
            && mainWindow()->currentTab()->url() == QUrl()) {
            newWindow = mainWindow();
        } else {
            newWindow = newMainWindow();
        }
        newWindow->restoreState(windows.at(i));
    }
}

bool BrowserApplication::isTheOnlyBrowser() const
{
    return (m_localServer != 0);
}

void BrowserApplication::installTranslator(const QString &name)
{
    QTranslator *translator = new QTranslator(this);  
    translator->load(name,QApplication::applicationDirPath());
    QApplication::installTranslator(translator);
}

#if defined(Q_WS_MAC)
bool BrowserApplication::event(QEvent* event)
{
    switch (event->type()) {
    case QEvent::ApplicationActivate: {
        clean();
        if (!m_mainWindows.isEmpty()) {
            BrowserMainWindow *mw = mainWindow();
            if (mw && !mw->isMinimized()) {
                mainWindow()->show();
            }
            return true;
        }
    }
    case QEvent::FileOpen:
        if (!m_mainWindows.isEmpty()) {
            mainWindow()->loadPage(static_cast<QFileOpenEvent *>(event)->file());
            return true;
        }
    default:
        break;
    }
    return QApplication::event(event);
}
#endif

void BrowserApplication::openUrl(const QUrl &url)
{
    mainWindow()->loadPage(url.toString());
}

BrowserMainWindow *BrowserApplication::newMainWindow()
{  
    BrowserMainWindow *browser = new BrowserMainWindow();    
    m_mainWindows.prepend(browser);
    browser->show();
    return browser;
}

BrowserMainWindow *BrowserApplication::mainWindow()
{
    clean();
    if (m_mainWindows.isEmpty())
        newMainWindow();
    return m_mainWindows[0];
}

void BrowserApplication::newLocalSocketConnection()
{
    QLocalSocket *socket = m_localServer->nextPendingConnection();
    if (!socket)
        return;
    socket->waitForReadyRead(1000);
    QTextStream stream(socket);
    QString url;
    stream >> url;
    if (!url.isEmpty()) {        
        QString appPath = QApplication::applicationDirPath ()+"/config.ini"; 
        QSettings settings(appPath,QSettings::IniFormat); 
        settings.beginGroup(QLatin1String("general"));
        int openLinksIn = settings.value(QLatin1String("openLinksIn"), 0).toInt();
        settings.endGroup();
       
        if (openLinksIn == 1)
            newMainWindow();
        else
            mainWindow()->tabWidget()->newTab();
        openUrl(url);
    }
    if (socket) delete socket;
    mainWindow()->raise();
    mainWindow()->activateWindow();
}

CookieJar *BrowserApplication::cookieJar()
{
    return (CookieJar*)networkAccessManager()->cookieJar();
}

DownloadManager *BrowserApplication::downloadManager()
{
    if (!s_downloadManager) {
        s_downloadManager = new DownloadManager();
    }
    return s_downloadManager;
}

NetworkAccessManager *BrowserApplication::networkAccessManager()
{
    if (!s_networkAccessManager) {
        s_networkAccessManager = new NetworkAccessManager();
        s_networkAccessManager->setCookieJar(new CookieJar);
    }
    return s_networkAccessManager;
}

HistoryManager *BrowserApplication::historyManager()
{
    if (!s_historyManager) {
        s_historyManager = new HistoryManager();
        QWebHistoryInterface::setDefaultInterface(s_historyManager);
    }
    return s_historyManager;
}

BookmarksManager *BrowserApplication::bookmarksManager()
{
    if (!s_bookmarksManager) {
        s_bookmarksManager = new BookmarksManager;
    }
    return s_bookmarksManager;
}

QIcon BrowserApplication::icon(const QUrl &url) const
{
    QIcon icon = QWebSettings::iconForUrl(url);
    if (!icon.isNull())
        return icon.pixmap(16, 16);
    if (m_defaultIcon.isNull())
        m_defaultIcon = QIcon(QLatin1String(":defaulticon.png"));
    return m_defaultIcon.pixmap(16, 16);
}

/** Slot: Called when the Tor process is started. It will connect the control
 * socket and set the icons and tooltips accordingly. */
void  BrowserApplication::started()
{
  TorSettings settings;

  updateTorStatus(Started);

  /* Now that Tor is running, we want to know if it dies when we didn't want
   * it to. */
  _isIntentionalExit = false;
  /* We haven't started a delayed shutdown yet. */
  _delayedShutdownStarted = false;
  /* Remember whether we started Tor or not */
  _isTorNavigatorRunningTor = _torControl->isTorNavigatorRunningTor();
  /* Try to connect to Tor's control port */
  _torControl->Connect(settings.getControlAddress(),settings.getControlPort());
  setStartupProgress(STARTUP_PROGRESS_CONNECTING, tr("Connecting to Tor"));
}

/** Sets the visibility of the startup status description and progress bar to
 * <b>visible</b>. */
void BrowserApplication::setStartupProgressVisible(bool visible)
{
  /* XXX: We force a repaint() to make sure the progress bar and onion status
   * icon don't overlap briefly. This is pretty hacktastic. */
  if (visible) {
//    ui.lblTorStatus->setVisible(false);
//    ui.lblTorStatusImg->setVisible(false);
//    repaint(ui.grpStatus->rect());
//    ui.lblStartupProgress->setVisible(true);
//    ui.progressBar->setVisible(true);
//      mainWindow()->m_progressBar->setVisible(true);
  } else {
//    ui.lblStartupProgress->setVisible(false);
//    ui.progressBar->setVisible(false);
//      mainWindow()->m_progressBar->setVisible(false);
//    repaint(ui.grpStatus->rect());
//    ui.lblTorStatus->setVisible(true);
//    ui.lblTorStatusImg->setVisible(true);
  }
}

/** Sets the progress bar completion value to <b>progressValue</b> and sets
 * the status text to <b>description</b>. */
void  BrowserApplication::setStartupProgress(int progressValue,const QString &description)
{
 // ui.progressBar->setValue(progressValue);
 mainWindow()->m_progressBar->setValue(progressValue);
 mainWindow()->m_progressBar->setToolTip(description);
 // ui.lblStartupProgress->setText(description);
 // _trayIcon.setToolTip(description);
}

/** Called when the Tor process fails to start, for example, because the path
 * specified to the Tor executable didn't lead to an executable. */
void BrowserApplication::startFailed(QString errmsg)
{
  /* We don't display the error message for now, because the error message
   * that Qt gives us in this instance is almost always "Unknown Error". That
   * will make users sad. */
  Q_UNUSED(errmsg);
 
  updateTorStatus(Stopped);

  /* Display an error message and see if the user wants some help */
  int response = VMessageBox::warning(0, tr("Error Starting Tor"),
                   tr("Tor Navigator was unable to start Tor. Check your settings "
                        "to ensure the correct name and location of your Tor "
                        "executable is specified."),
                   VMessageBox::ShowSettings|VMessageBox::Default,
                   VMessageBox::Cancel|VMessageBox::Escape,
                   VMessageBox::Help);

  if (response == VMessageBox::ShowSettings) {
    /* Show the settings dialog so the user can make sure they're pointing to
     * the correct Tor. */
    //showConfigDialog();
  } else if (response == VMessageBox::Help) {
    /* Show troubleshooting information about starting Tor */
    //showHelpDialog("troubleshooting.start");
  }
}



//////////////////<
/** Called when the control socket has successfully connected to Tor. */
void
BrowserApplication::connected()
{
  authenticate();
}

/** Called when TorNavigator wants to disconnect from a Tor it did not start. */
void
BrowserApplication::disconnect()
{
  _torControl->disconnect();
}

/** Called when the control socket has been disconnected. */
void
BrowserApplication::disconnected()
{
  if (!_isTorNavigatorRunningTor) {
    /* If we didn't start our own Tor process, interpret losing the control
     * connection as "Tor is stopped". */
    updateTorStatus(Stopped);
  }
  
  /*XXX We should warn here if we get disconnected when we didn't intend to */
  //_newIdentityAct->setEnabled(false);
  //ui.lblNewIdentity->setEnabled(false);
  _isTorNavigatorRunningTor = false;  
  
  
   mainWindow()->m_torAction->setEnabled(false);
   mainWindow()-> m_verifyTorAction->setEnabled(false);
   mainWindow()->m_changeIdentityAction->setEnabled(false);
   int value=(mainWindow()->m_progressBar->minimum());
   setStartupProgress(value,tr("Not connected to the Tor network!"));
}

/** Attempts to authenticate to Tor's control port, depending on the
 * authentication method specified in TorSettings::getAuthenticationMethod().
 */
bool
BrowserApplication::authenticate()
{
  TorSettings::AuthenticationMethod authMethod;
  TorSettings settings;
  ProtocolInfo pi;
  
  updateTorStatus(Authenticating);
  setStartupProgress(STARTUP_PROGRESS_AUTHENTICATING,
                     tr("Authenticating to Tor"));

  authMethod = settings.getAuthenticationMethod(); 
  pi = _torControl->protocolInfo();
  if (!pi.isEmpty())
  	
  	 {
    QStringList authMethods = pi.authMethods();
    if (authMethods.contains("COOKIE"))
      authMethod = TorSettings::CookieAuth;
    else if (authMethods.contains("HASHEDPASSWORD"))
      authMethod = TorSettings::PasswordAuth;
    else if (authMethods.contains("NULL"))
      authMethod = TorSettings::NullAuth;
     }
  
  if (authMethod == TorSettings::CookieAuth) {
    /* Try to load an auth cookie and send it to Tor */
    QByteArray cookie = loadControlCookie(pi.cookieAuthFile());
    while (cookie.isEmpty()) {
      // Prompt the user to find their control_auth_cookie 
      int ret = VMessageBox::question(0,
                  tr("Cookie Authentication Required"),
                  p(tr("The Tor software requires Vidalia to send the "
                       "contents of an authentication cookie, but Vidalia "
                       "was unable to find one."))
                  + p(tr("Would you like to browse for the file "
                         "'control_auth_cookie' yourself?")),
                VMessageBox::Browse|VMessageBox::Default,
                VMessageBox::Cancel|VMessageBox::Escape);
      
      if (ret == VMessageBox::Cancel)
        goto cancel;
      QString cookieDir = QFileDialog::getOpenFileName(0,
                            tr("Data Directory"),
                            settings.getDataDirectory(),
                            tr("Control Cookie (control_auth_cookie)"));
      if (cookieDir.isEmpty())
        goto cancel;
      cookieDir = QFileInfo(cookieDir).absolutePath();
      cookie = loadControlCookie(cookieDir);
    }
    
    vNotice("Authenticating using 'cookie' authentication.");
    return _torControl->authenticate(cookie);
  }
 
   else if (authMethod == TorSettings::PasswordAuth) {
    // Get the control password and send it to Tor 
    vNotice("Authenticating using 'hashed password' authentication.");
    if (_useSavedPassword) {
      TorSettings settings;
      _controlPassword = settings.getControlPassword();
    }
    return _torControl->authenticate(_controlPassword);
  }
  // No authentication. Send an empty password. 
  vNotice("Authenticating using 'null' authentication.");
  return _torControl->authenticate(QString(""));

cancel:
  vWarn("Cancelling control authentication attempt.");
  if (_isTorNavigatorRunningTor)
    stop();
  else
    disconnect();
  return false;
}

/** Called when TorNavigator has successfully authenticated to Tor. */
void
BrowserApplication::authenticated()
{
  ServerSettings serverSettings(_torControl);
  QString errmsg;

  updateTorStatus(Authenticated);
  
  /* If Tor doesn't have bootstrapping events, then update the current
   * status string and bump the progress bar along a bit. */
  if (_torControl->getTorVersion() < 0x020101) {
    setStartupProgress(STARTUP_PROGRESS_CIRCUITBUILD,tr("Connecting to the Tor network"));
  }
  
  /* Let people click on their beloved "New Identity" button */
//  _newIdentityAct->setEnabled(true);
//  ui.lblNewIdentity->setEnabled(true);

  /* Register for any pertinent asynchronous events. */
  if (!_torControl->setEvents(&errmsg)) {
    VMessageBox::warning(0, tr("Error Registering for Events"),
      (tr("Tor Navigator was unable to register for some events. "
           "Many of Tor Navigator's features may be unavailable."))
         + (errmsg),
      VMessageBox::Ok);      
      
  }

  /* Configure UPnP port forwarding if needed */
  serverSettings.configurePortForwarding();

  /* Check if Tor has a circuit established */
  if (_torControl->circuitEstablished()) circuitEstablished();
  /* Check the status of Tor's version */
  if (_torControl->getTorVersion() >= 0x020001)
    checkTorVersion();
  if (_torControl->getTorVersion() >= 0x020102) {
    BootstrapStatus status = _torControl->bootstrapStatus();
    if (status.isValid())
      bootstrapStatusChanged(status);
    }
}



/** Called when Tor has successfully established a circuit. */
void BrowserApplication::circuitEstablished()
{
    updateTorStatus(CircuitEstablished);    

    int value=(mainWindow()->m_progressBar->maximum());
    setStartupProgress(value,tr("Connected to the Tor network!"));
    mainWindow()->m_torAction->setEnabled(true);
    mainWindow()-> m_verifyTorAction->setEnabled(true);
    mainWindow()->m_changeIdentityAction->setEnabled(true);
   


/**  StartSubprocesses() */

#if defined(USE_AUTOUPDATE)
  VidaliaSettings settings;
  if (settings.isAutoUpdateEnabled()) {
    QDateTime lastCheckedAt = settings.lastCheckedForUpdates();
    if (UpdateProcess::shouldCheckForUpdates(lastCheckedAt)) {
      /* Initiate a background check for updates now */
      _updateTimer.stop();
      checkForUpdates();
    }
  }
#endif
}


/** Called when Vidalia fails to authenticate to Tor. The failure reason is
 * specified in <b>errmsg</b>. */
void BrowserApplication::authenticationFailed(QString errmsg)
{
  bool retry = false;
  
  vWarn("Authentication failed: %1").arg(errmsg); 


 /* Parsing log messages is evil, but we're left with little option */
  if (errmsg.contains("Password did not match")) {
    ControlPasswordInputDialog dlg;
    connect(&dlg, SIGNAL(helpRequested(QString)),
            this, SLOT(showHelpDialog(QString)));

  qint64 torPid = 0;   

#if defined(Q_OS_WIN32)
   
    QHash<qint64, QString> procs = process_list();
    foreach (qint64 pid, procs.keys()) {
      if (! procs.value(pid).compare("tor.exe", Qt::CaseInsensitive)) {
        torPid = pid;
        break;
      }
    }
  // dlg.setResetEnabled(torPid > 0);
#else
  //dlg.setResetEnabled(false);
#endif

   int ret = dlg.exec();
    if (ret == QDialogButtonBox::Ok) {
      if (dlg.isSavePasswordChecked()) {
        TorSettings settings;
        settings.setAuthenticationMethod(TorSettings::PasswordAuth);
        settings.setUseRandomPassword(false);
        settings.setControlPassword(dlg.password());
        _useSavedPassword = true;
      } else {
        _controlPassword = dlg.password();
        _useSavedPassword = false;
      }
      retry = true;
    } else if (ret == QDialogButtonBox::Reset) {
      if (! process_kill(torPid)) {
        VMessageBox::warning(0,
          tr("Password Reset Failed"),
          p(tr("TorNavigator tried to reset Tor's control password, but was not "
               "able to restart the Tor software. Please check your Task "
               "Manager to ensure there are no other Tor processes running.")),
               VMessageBox::Ok|VMessageBox::Default);
      } else {
        retry = true;
      }
    }
  } else
   {
    /* Something else went wrong */
   
    int ret = VMessageBox::warning(0, 
                tr("Authentication Error"),
                p(tr("TorNavigator was unable to authenticate to the Tor software. "
                     "(%1)").arg(errmsg)) + 
                p(tr("Please check your control port authentication "
                     "settings.")),
                VMessageBox::ShowSettings|VMessageBox::Default,
                VMessageBox::Cancel|VMessageBox::Escape);
    
   if (ret == VMessageBox::ShowSettings)
   {
   	SettingsDialog *dlg = new SettingsDialog(mainWindow());
    dlg->show();
  	}   	
 
  }
  
  if (_torControl->isRunning())
    if (_isTorNavigatorRunningTor) 
     stop();
    else
      disconnect();
  if (retry)
    startTor();
}

/** Called when the connection to the control socket fails. The reason will be
 * given in the errmsg parameter. */
void BrowserApplication::connectFailed(QString errmsg)
{
  /* Ok, ok. It really isn't going to connect. I give up. */
  int response = VMessageBox::warning(0, 
                   tr("Connection Error"), (errmsg),
                   VMessageBox::Ok|VMessageBox::Default|VMessageBox::Escape, 
                   VMessageBox::Retry);


  if (response == VMessageBox::Retry) {
    /* Let's give it another try. */
    TorSettings settings;
    _torControl->Connect(settings.getControlAddress(),settings.getControlPort());
  } else {
   
    /* Since TorNavigator can't connect, we can't really do much, so stop Tor. */
    _torControl->stop();
  }
}

/** Disconnects the control socket and stops the Tor process. */
bool BrowserApplication::stop()
{
  ServerSettings server(_torControl);
  QString errmsg;
  TorStatus prevStatus;
  bool rc;

  /* If we're running a server, give users the option of terminating
   * gracefully so clients have time to find new servers. */
  if (serverEnabled)/* && !_delayedShutdownStarted)*/{ 
  //if (server.isServerEnabled() && !_delayedShutdownStarted) {
    /* Ask the user if they want to shutdown nicely. */
    int response = VMessageBox::question(0, tr("Relaying is Enabled"),
                     tr("You are currently running a relay. "
                        "Terminating your relay will interrupt any "
                        "open connections from clients.\n\n"
                        "Would you like to shutdown gracefully and "
                        "give clients time to find a new relay?"),
                        VMessageBox::Yes|VMessageBox::Default, 
                        VMessageBox::No);
    if (response == VMessageBox::Yes)
      _delayedShutdownStarted = true;
    else 
      return false;
  }
  
  prevStatus = updateTorStatus(Stopping);  
  if (_delayedShutdownStarted) {
    /* Start a delayed shutdown */
    rc = _torControl->signal(TorSignal::Shutdown, &errmsg);
  } else {
    /* We want Tor to stop now, regardless of whether we're a server. */
    _isIntentionalExit = true;
    rc = _torControl->stop(&errmsg);
  }
  
  if (!rc) {
    /* We couldn't tell Tor to stop, for some reason. */
    /*int response =*/ VMessageBox::warning(0, tr("Error Shutting Down"),
                     (tr("TorNavigator was unable to stop the Tor software.")) 
                       + (errmsg),
                     VMessageBox::Ok); 
                    
    /* Tor is still running since stopping failed */
    _isIntentionalExit = false;
    _delayedShutdownStarted = false;
    updateTorStatus(prevStatus);
  }
  return rc;
}

/** Updates the UI to reflect Tor's current <b>status</b>. Returns the
 * previously set TorStatus value.*/
BrowserApplication::TorStatus
BrowserApplication::updateTorStatus(TorStatus status)
{
  QString statusText, actionText;
  QString trayIconFile, statusIconFile;
  TorStatus prevStatus = _status;
 
 // vNotice("Tor status changed from '%1' to '%2'.").arg(toString(prevStatus)).arg(toString(status)); 
  _status = status;

  if (status == Stopped) {
      statusText = tr("Tor is not running");
      actionText = tr("Start Tor");
      
      setStartupProgressVisible(false);
  } else if (status == Stopping) {
      if (_delayedShutdownStarted) {
        statusText = tr("Your relay is shutting down.\n" 
                        "Click 'Stop' again to stop your relay now.");
      } else {
        statusText = tr("Tor is shutting down");
      }

  } else if (status == Started) {
      actionText = tr("Stop Tor");
  
  } else if (status == Starting)  {
      statusText = tr("Starting the Tor software");
    
      setStartupProgressVisible(true);
      setStartupProgress(STARTUP_PROGRESS_STARTING, statusText);
  } else if (status == CircuitEstablished) {
      statusText = tr("Connected to the Tor network!");     
      setStartupProgressVisible(false);
  }
  
   return prevStatus;

}


bool BrowserApplication::slotChangeIdentity()
{
	if (_torControl->changeIdentity())
	{
	return true;	
	}	
	return false;
}

/** Slot: Called when the Tor process has exited. It will adjust the tray
 * icons and tooltips accordingly. */
void 
 BrowserApplication::stopped(int exitCode, QProcess::ExitStatus exitStatus)
{
  updateTorStatus(Stopped);

  /* If we didn't intentionally close Tor, then check to see if it crashed or
   * if it closed itself and returned an error code. */
  if (!_isIntentionalExit) {
    /* A quick overview of Tor's code tells me that if it catches a SIGTERM or
     * SIGINT, Tor will exit(0). We might need to change this warning message
     * if this turns out to not be the case. */
    if (exitStatus == QProcess::CrashExit || exitCode != 0) {
      int ret = VMessageBox::warning(0, tr("Unexpected Error"),
                  tr("Tor Navigator detected that the Tor software exited "
                     "unexpectedly.\n\n"
                     "Please check the message log for recent "
                     "warning or error messages."),
                  VMessageBox::Ok|VMessageBox::Escape, 
                  VMessageBox::ShowLog|VMessageBox::Default);                 
      if (ret == VMessageBox::ShowLog)         
         mainWindow()->m_messageLog->show();  
     
    }
  }
}

/** Searches for and attempts to load the control authentication cookie. This
 * assumes the cookie is named 'control_auth_cookie'. If <b>cookiePath</b> is
 * empty, this method will search some default locations depending on the
 * current platform. <b>cookiePath</b> can point to either a cookie file or a
 * directory containing the cookie file. */
QByteArray BrowserApplication::loadControlCookie(QString cookiePath)
{
  QFile authCookie;
  QStringList pathList;

  if (!cookiePath.isEmpty())
  	 {
    pathList << cookiePath;
     } else
     {
    /* Try some default locations */
    TorSettings settings;
    QString dataDir = settings.getDataDirectory();
    if (!dataDir.isEmpty())
      pathList << dataDir;
      
#if defined(Q_WS_WIN)
    pathList << expand_filename("%APPDATA%\\Tor");
#else
    pathList << expand_filename("~/.tor");
#endif
    }
    
    /* Search for the cookie file */
  foreach (QString path, pathList) {
    QString cookieFile = QFileInfo(path).isFile() ?
                          path : path + "/control_auth_cookie";
    vDebug("Checking for authentication cookie in '%1'").arg(cookieFile);
    if (!QFileInfo(cookieFile).exists())
      continue;
    
    authCookie.setFileName(cookieFile);
    if (authCookie.open(QIODevice::ReadOnly)) {
      vInfo("Reading authentication cookie from '%1'").arg(cookieFile);
      return authCookie.readAll();
    } else {
      vWarn("Couldn't open cookie file '%1': %2")
        .arg(cookieFile).arg(authCookie.errorString());
    }
  }
  vWarn("Couldn't find a readable authentication cookie.");
  return QByteArray();
}

/** Checks the status of the current version of Tor to see if it's old,
 * unrecommended, or obsolete. */
void BrowserApplication::checkTorVersion()
{
  QString status;
  if (_torControl->getInfo("status/version/current", status))
  	 {
    if (!status.compare("old", Qt::CaseInsensitive)
          || !status.compare("unrecommended", Qt::CaseInsensitive)
          || !status.compare("obsolete", Qt::CaseInsensitive))
           {
      dangerousTorVersion();
           }
     }
}

/** Called when Tor's bootstrapping status changes. <b>bse</b> represents
 * Tor's current estimate of its bootstrapping progress. */
void BrowserApplication::bootstrapStatusChanged(const BootstrapStatus &bs)
{
  int percentComplete = STARTUP_PROGRESS_BOOTSTRAPPING + bs.percentComplete();
  bool warn = (bs.severity() == tc::SeverityWarn && 
               bs.recommendedAction() != BootstrapStatus::RecommendIgnore);

  QString description;
  switch (bs.status()) {
    case BootstrapStatus::ConnectingToDirMirror:
      description = tr("Connecting to a relay directory");
      break;
    case BootstrapStatus::HandshakingWithDirMirror:
    case BootstrapStatus::CreatingOneHopCircuit:
      description = tr("Establishing an encrypted directory connection");
      break;
    case BootstrapStatus::RequestingNetworkStatus:
      description = tr("Retrieving network status");
      break;
    case BootstrapStatus::LoadingNetworkStatus:
      description = tr("Loading network status");
      break;
    case BootstrapStatus::LoadingAuthorityCertificates:
      description = tr("Loading authority certificates");
      break;
    case BootstrapStatus::RequestingDescriptors:
      description = tr("Requesting relay information");
      break;
    case BootstrapStatus::LoadingDescriptors:
      description = tr("Loading relay information");
      break;
    case BootstrapStatus::ConnectingToEntryGuard:
      description = tr("Connecting to the Tor network");
      break;
    case BootstrapStatus::HandshakingWithEntryGuard:
    case BootstrapStatus::EstablishingCircuit:
      description = tr("Establishing a Tor circuit");
      break;
    case BootstrapStatus::BootstrappingDone:
      description = tr("Connected to the Tor network!");
      warn = false; /* probably false anyway */
      break;
    default:
      description = tr("Unrecognized startup status");
  }
  if (warn) {
    QString reason;
    /* Is it really a good idea to translate these? */
    switch (bs.reason()) {
      case tc::MiscellaneousReason:
        reason = tr("miscellaneous");
        break;
      case tc::IdentityMismatch:
        reason = tr("identity mismatch");
        break;
      case tc::ConnectionDone:
        reason = tr("done");
        break;
      case tc::ConnectionRefused:
        reason = tr("connection refused");
        break;
      case tc::ConnectionTimeout:
        reason = tr("connection timeout");
        break;
      case tc::ConnectionIoError:
        reason = tr("read/write error");
        break;
      case tc::NoRouteToHost:
        reason = tr("no route to host");
        break;
      case tc::ResourceLimitReached:
        reason = tr("insufficient resources");
        break;
      default:
        reason = tr("unknown");
    }
    description += tr(" failed (%1)").arg(reason);
  }
  setStartupProgress(percentComplete, description);
}

/** Called when Tor thinks its version is old or unrecommended, and displays a
 * message notifying the user. */
void BrowserApplication::dangerousTorVersion()
{
  static bool alreadyWarned = false;

  if (!alreadyWarned) {
#if !defined(USE_AUTOUPDATE)
    QString website = "https://www.torproject.org/";
# if QT_VERSION >= 0x040200
    website = QString("<a href=\"%1\">%1</a>").arg(website);
# endif

    VMessageBox::information(0, tr("Tor Update Available"),
      p(tr("The currently installed version of Tor is out of date or no longer "
           "recommended. Please visit the Tor website to download the latest "
           "version.")) + p(tr("Tor website: %1").arg(website)),
      VMessageBox::Ok);
#else
    int ret = VMessageBox::information(this,
                tr("Tor Update Available"),
                p(tr("The currently installed version of Tor is out of date "
                     "or no longer recommended."))
                  + p(tr("Would you like to check if a newer package is "
                         "available for installation?")),
                VMessageBox::Yes|VMessageBox::Default,
                VMessageBox::No|VMessageBox::Escape);

    if (ret == VMessageBox::Yes)
      checkForUpdatesWithUI();
#endif
    alreadyWarned = true;
  }
}

/** Called when the application has started and the main event loop is
 * running. */
void BrowserApplication::mainLoopRunning()
{
}

/** Terminate the Tor process if it is being run under Vidalia, disconnect all
 * TorControl signals, and exit Vidalia. */
void BrowserApplication::aboutToQuit()
{
		
  vNotice("Cleaning up before exiting.");

  if (_torControl->isTorNavigatorRunningTor())
  	 {
    /* Kill our Tor process now */ 
    _torControl->stop();
     }

  /* Disable port forwarding */
  ServerSettings settings(_torControl);
  settings.cleanupPortForwarding();

  /* Disconnect all of the TorControl object's signals */
  QObject::disconnect(_torControl, 0, 0, 0);
}

/** Writes <b>msg</b> with severity <b>level</b> to Vidalia's log. */
Log::LogMessage
BrowserApplication::log(Log::LogLevel level, QString msg)
{
		
	
  return _log.log(level, msg);
}

/** Catches debugging messages from Qt and sends them to Vidalia's logs. If Qt
 * emits a QtFatalMsg, we will write the message to the log and then abort().
 */
void
BrowserApplication::qt_msg_handler(QtMsgType type, const char *s)
{
	

	
  QString msg(s);
  switch (type) {
    case QtDebugMsg:   
    	vDebug("QtDebugMsg: %1").arg(msg); 
      break;
    case QtWarningMsg:
      vNotice("QtWarningMsg: %1").arg(msg);
      break;
    case QtCriticalMsg:
      vWarn("QtCriticalMsg: %1").arg(msg);
      break;
    case QtFatalMsg:
      vError("QtFatalMsg: %1").arg(msg);
      break;
  }
  if (type == QtFatalMsg) {
    vError("Fatal Qt error. Aborting.");
    abort();
  }
}


/** Catches and processes Tor client and general status events. */
void
BrowserApplication::customEvent(QEvent *event)
{
  if (event->type() == CustomEventType::ClientStatusEvent) {
    ClientStatusEvent *cse = dynamic_cast<ClientStatusEvent *>(event);
    if (!cse)
      return;

    if (cse->status() == ClientStatusEvent::CircuitEstablished) {
      circuitEstablished();
      cse->accept();
    } else if (cse->status() == ClientStatusEvent::Bootstrap) {
      BootstrapStatusEvent *bse = dynamic_cast<BootstrapStatusEvent *>(cse);
      if (bse)
        bootstrapStatusChanged(bse->status());
      cse->accept();
    } else if (cse->status() == ClientStatusEvent::DangerousPort) {
      DangerousPortEvent *dpe = dynamic_cast<DangerousPortEvent *>(cse);
      if (dpe) {
        bool rejected = (dpe->result() == DangerousPortEvent::Reject);
        warnDangerousPort(dpe->port(), rejected);
      }
      cse->accept();
    }
  } else if (event->type() == CustomEventType::GeneralStatusEvent) {
    GeneralStatusEvent *gse = dynamic_cast<GeneralStatusEvent *>(event);
    if (!gse)
      return;

    if (gse->status() == GeneralStatusEvent::DangerousTorVersion) {
      DangerousVersionEvent *dve = dynamic_cast<DangerousVersionEvent *>(gse);
      if (dve && (dve->reason() == DangerousVersionEvent::ObsoleteVersion
           || dve->reason() == DangerousVersionEvent::UnrecommendedVersion)) {
        dangerousTorVersion();
      }
      gse->accept();
    }
  }
}

/** Called when Tor thinks the user has tried to connect to a port that
 * typically is used for unencrypted applications. Warns the user and allows
 * them to ignore future warnings on <b>port</b>. */
void
BrowserApplication::warnDangerousPort(quint16 port, bool rejected)
{
  QString warning, application;
  QMessageBox dlg(QMessageBox::Warning,
                  tr("Potentially Unsafe Connection"), QString(),
                  QMessageBox::Ok | QMessageBox::Ignore);

  switch (port) {
    case  23:
     application = tr(", probably Telnet, ");
     break;

    case 109:
    case 110:
    case 143:
      application = tr(", probably an email client, "); 
      break;

    default:
      application = " ";
  }

  warning = p(tr("One of your applications%1appears to be making a "
                 "potentially unencrypted and unsafe connection to port %2. "
                 "Anything sent over this connection could be monitored. "
                 "Please check your application's configuration and use "
                 "only encrypted protocols, such as SSL, if possible.")
                 .arg(application).arg(port));
  if (rejected) {
    warning.append(p(tr("Tor has automatically closed your connection in "
                        "order to protect your anonymity.")));
  }
  dlg.setText(warning);

  int ret = dlg.exec();
  if (ret == QMessageBox::Ignore) {
    TorSettings settings;
    QList<quint16> ports;
    int idx;

    ports = settings.getWarnPlaintextPorts();
    idx   = ports.indexOf(port);
    if (idx >= 0) {
      ports.removeAt(idx);
      settings.setWarnPlaintextPorts(ports);
    }

    ports = settings.getRejectPlaintextPorts();
    idx   = ports.indexOf(port);
    if (idx >= 0) {
      ports.removeAt(idx);
      settings.setRejectPlaintextPorts(ports);
    }
  }
}

void
BrowserApplication::loadDefaultCaCertificates() const
{
  QSslSocket::setDefaultCaCertificates(QList<QSslCertificate>());

  if (! QSslSocket::addDefaultCaCertificates(":/pki/cacert_root.crt"))
    vWarn("Failed to add the GeoIP CA certificate to the default CA "
          "certificate database.");

  if (! QSslSocket::addDefaultCaCertificates(":/pki/EntrustSecureServerCA.crt"))
    vWarn("Failed to add the Entrust Secure Server CA certificate to the "
          "default CA certificate database.");

  if (! QSslSocket::addDefaultCaCertificates(":/pki/gd-class2-root.crt"))
    vWarn("Failed to add the GoDaddy Class 2 CA certificate to the "
          "default CA certificate database.");
}

// start tor
void BrowserApplication::startTor()
{
  torrcPath  =  QApplication::applicationDirPath ()+"/tor/linux/torrc";
  torExePath =  QApplication::applicationDirPath ()+"/tor/linux/tor";
  
  #if defined(Q_OS_WIN32)
  torrcPath  =  QApplication::applicationDirPath ()+"/tor/win32/torrc";
  torExePath =  QApplication::applicationDirPath ()+"/tor/win32/tor.exe"; 
  #endif   
   
  QString configPath = QApplication::applicationDirPath ()+"/config.ini";    
  QSettings settings(configPath,QSettings::IniFormat);
  settings.beginGroup(QLatin1String("Tor"));  
  int controlPort= settings.value(QLatin1String("controlPort"),9051).toInt();
  int authenticationMethod =settings.value(QLatin1String("authenticationMethod"),"none").toInt();
  controlPassword=settings.value(QLatin1String("controlPassword"),"[~xONM|aqrxk\"*LH").toString();  
  dataDirectory=QApplication::applicationDirPath ()+"/tor";
  hashedPassword=settings.value(QLatin1String("hashedPassword"),"").toString();
  serverEnabled=settings.value(QLatin1String("serverEnabled"),false).toBool();
  strictExitPolicyEnabled=settings.value(QLatin1String("strictExitPolicyEnabled"),false).toBool();  
  settings.endGroup();
  
    
  
  
   bool preferredMode=false;
   bool bannedMode=false;
   
   // if file exists, get preferred countries  
   QStringList preferredISOList;
   QStringList preferredList;
   QString preferredCountriesStr;
   QFile preferred(QApplication::applicationDirPath()+"/tor/preferred_countries"); 
   if (preferred.exists())
    {
    	preferredMode=true;
    	preferred.open(QIODevice::ReadOnly | QIODevice::Text);
    	
    	QTextStream in(&preferred);
        while (!in.atEnd())
        	 {
        		QString line = in.readLine();
        		preferredList.append(line);
        		if (CountryInfo::countryToISO(line)!="0")
        		{
        		line="{"+CountryInfo::countryToISO(line)+"}";
        		preferredISOList.append(line);
       		    }
             }
 	    preferred.close();
 	    mainWindow()->m_torDialog->torLogAddMsg(tr("Preferred countries = %1").arg(preferredList.join(",")));
 	    preferredCountriesStr=preferredISOList.join(",");
	}
	else
	{
	mainWindow()->m_torDialog->torLogAddMsg(tr("Preferred countries = all (No preference policy defined)"));	
	}     
     
    
   // if file exists,get banned countries 
   QStringList bannedISOList;
   QStringList bannedList;
   QString bannedCountriesStr;
   QFile banned(QApplication::applicationDirPath()+"/tor/banned_countries"); 
   if (banned.exists())
    {
    	bannedMode=true;
    	banned.open(QIODevice::ReadOnly | QIODevice::Text);
    	
    	QTextStream in(&banned);
        while (!in.atEnd())
        	 {
        		QString line = in.readLine();
        		bannedList.append(line);
        		if (CountryInfo::countryToISO(line)!="0")
        		{
        		line="{"+CountryInfo::countryToISO(line)+"}";
        		bannedISOList.append(line);	
       			}
        		
             }
 	    banned.close();
 	    mainWindow()->m_torDialog->torLogAddMsg(tr("Banned countries = %1").arg(bannedList.join(",")));
 	    bannedCountriesStr=bannedISOList.join(",");
	}
	else
	{
	mainWindow()->m_torDialog->torLogAddMsg(tr("Preferred countries = all (No preference policy defined)"));	
	} 
     
     //bannedCountriesStr.chop(1);
 
  QStringList args;
  
  if (hashedPassword.isEmpty()) 
  	{
  	hashedPassword=_torControl->hashPassword(controlPassword);
  	
    QString configPath = QApplication::applicationDirPath ()+"/config.ini";    
    QSettings settings(configPath,QSettings::IniFormat);
    settings.beginGroup(QLatin1String("tor"));
    settings.setValue(QLatin1String("hashedPassword"),hashedPassword);
    settings.endGroup();		
    }
    
  if (authenticationMethod==1)  	
  	args<<"HashedControlPassword"<<hashedPassword.toLatin1(); 
  	 
  if (!torrcPath.isEmpty())    
  	 args << "-f"<< torrcPath; 	   
  if (controlPort)
     args << "ControlPort" << QString::number(controlPort);  
  if (!dataDirectory.isEmpty())   
     args <<"DataDirectory" <<dataDirectory;
  if (!serverEnabled) 	
  	 {
  	   args<<"ClientOnly"<<"1";  	 	
  	   mainWindow()->m_torDialog->torLogAddMsg(tr("Starting in client only - server mode disabled"));
 	 }
 	 else
 	 {
 	 mainWindow()->m_torDialog->torLogAddMsg(tr("Excellent ! You're starting in server mode - server enabled"));
 	 serverEnabled=true; 	 
   	 }
  
  if (preferredMode||bannedMode)
  {
  	if (strictExitPolicyEnabled)
  	{
  		args<<"StrictExitNodes"<<"1";
  		mainWindow()->m_torDialog->torLogAddMsg(tr("Strict exit node policy enabled"));  		
 	}
 	
 	 if (preferredMode)
 	{
 	  args<<"ExitNodes";
 	  args<<preferredCountriesStr;	
	}
	
	if (bannedMode)
	{
	  args<<"ExcludeNodes";
	  args<<bannedCountriesStr;
	}	
  		
  }
 
   _torControl->start(torExePath, args);
   emit running();	
}

void BrowserApplication::updatePrivoxyConfig()
{   
    QString line;
    QString privoxyConfigPath=QApplication::applicationDirPath();   
    
    privoxyConfigPath=privoxyConfigPath+"/privoxy/linux/config";
    #if defined(Q_OS_WIN32)    
    privoxyConfigPath=privoxyConfigPath+"/privoxy/win32/config.txt";    
    #endif   
   
       
    QFile privoxyConfigFile(privoxyConfigPath);
    QString tempFilePath;
    
    tempFilePath=QApplication::applicationDirPath ()+"/privoxy/linux/privoxyconfig.temp";
    #if defined(Q_OS_WIN32)
    tempFilePath=QApplication::applicationDirPath ()+"/privoxy/win32/config.temp";
    #endif
    QFile tempFile(tempFilePath);
    
     if (!privoxyConfigFile.exists())
     QMessageBox::warning(0,tr("I/O Error :"),tr("Privoxy config file not found"));
    
     if (privoxyConfigFile.open(QIODevice::ReadWrite | QIODevice::Text))
     {     	
     	 tempFile.open(QIODevice::ReadWrite| QIODevice::Text);
     	 
     	 QTextStream in(&privoxyConfigFile);
     	 QTextStream out(&tempFile);
         while (!in.atEnd())
         	 {
        QString line = in.readLine();       
        if(line.startsWith("confdir"))
        		{
        		line="confdir "+ QApplication::applicationDirPath ()+"/privoxy/linux";        		
        		#if defined(Q_OS_WIN32) 
        	    line="confdir "+ QApplication::applicationDirPath ()+"/privoxy/win32";        	 
        	    #endif
        	   	}
         if(line.startsWith("logdir"))
        		{
        		line="logdir "+ QApplication::applicationDirPath ()+"/privoxy/linux";        		
        		#if defined(Q_OS_WIN32) 
        	    line="logdir "+ QApplication::applicationDirPath ()+"/privoxy/win32";        	 
        	    #endif
        	   	}
        out<<line<<"\n";         
         	    }         	  
     }
     else
     	     QMessageBox::warning(0,tr("I/O error :"),tr("cannot open privoxy config file")); 		
    		 		
    		 privoxyConfigFile.close();    		   		 
    		
    		 QFile::remove(privoxyConfigPath+"~");
    		 if 
    		 (privoxyConfigFile.remove(privoxyConfigPath))
    		 {
    		 tempFile.rename(tempFilePath,privoxyConfigPath);	
   		 	 }
    		 else
    		 QMessageBox::warning(0,tr("Error"),tr("cannot update privoxy config file"));
    		 tempFile.close();
   		 	
}







