/****************************************************************************
**
** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
** Contact: Qt Software Information (qt-info@nokia.com)
**
** This file is part of the demonstration applications of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain
** additional rights. These rights are described in the Nokia Qt LGPL
** Exception version 1.0, included in the file LGPL_EXCEPTION.txt in this
** package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you are unsure which license is appropriate for your use, please
** contact the sales department at qt-sales@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef BROWSERAPPLICATION_H
#define BROWSERAPPLICATION_H

#include <QtGui/QApplication>
#include <QtCore/QUrl>
#include <QtCore/QPointer>
#include <QProcess>
#include <QtGui/QIcon>
#include <QMap>
#include <QList>
#include <QString>
#include <QKeySequence>

#include "torcontrol.h"
#include "privoxycontroller.h"
#include "vidaliasettings.h"
#include "log.h"
#include "config.h"
#include "messagelog.h"
#include "bootstrapstatusevent.h"

/** Pointer to this TorNavigator application instance. */
#define vApp  (( BrowserApplication *)qApp)


#define vDebug(fmt)   (vApp->log(Log::Debug, (fmt)))
#define vInfo(fmt)    (vApp->log(Log::Info, (fmt)))
#define vNotice(fmt)  (vApp->log(Log::Notice, (fmt)))
#define vWarn(fmt)    (vApp->log(Log::Warn, (fmt)))
#define vError(fmt)   (vApp->log(Log::Error, (fmt)))

QT_BEGIN_NAMESPACE
class QLocalServer;
QT_END_NAMESPACE

class BookmarksManager;
class BrowserMainWindow;
class CookieJar;
class DownloadManager;
class HistoryManager;
class NetworkAccessManager;

class BrowserApplication : public QApplication
{
    Q_OBJECT

public:
    BrowserApplication(int &argc, char **argv);
    ~BrowserApplication();
    static BrowserApplication *instance();
    void loadSettings();
    
    bool isTheOnlyBrowser() const;
    BrowserMainWindow *mainWindow();
    QList<BrowserMainWindow*> mainWindows();
    QIcon icon(const QUrl &url) const;

    void saveSession();
    bool canRestoreSession() const;
    bool stop(); /** Called when the user selects "Stop" form the menu. */  

    static HistoryManager *historyManager();
    static CookieJar *cookieJar();
    static DownloadManager *downloadManager();
    static NetworkAccessManager *networkAccessManager();
    static BookmarksManager *bookmarksManager();    
    
    static bool readPasswordFromStdin();//Returns true if TorNavigator should read the control password from stdin
    
    QString torrcPath;
    QString torExePath;
    quint16 controlPort;
    int authenticationMethod ;
    QString controlPassword;
    QString dataDirectory;
    QString hashedPassword;
    bool serverEnabled;
    bool strictExitPolicyEnabled;
    
    // Writes <b>msg</b> with severity <b>level</b> to TorNavigator's log.
    static Log::LogMessage log(Log::LogLevel level, QString msg);
    /** Returns TorNavigator's main TorControl object. */
    static TorControl* torControl() { return _torControl; }
    
    signals:
  /** Emitted when the application is running and the main event loop has
   * started. */ 
  void running();   


#if defined(Q_WS_MAC)
    bool event(QEvent *event);
#endif

public slots:
    BrowserMainWindow *newMainWindow();
    void restoreLastSession();
#if defined(Q_WS_MAC)
    void lastWindowClosed();
    void quitBrowser();
#endif
    bool slotChangeIdentity();
    
    protected:
  /** Catches and processes Tor client status events. */
  virtual void customEvent(QEvent *event);
  /** Called when the user changes the UI translation. */
  //virtual void retranslateUi(); 

private slots:
  void postLaunch();
  void openUrl(const QUrl &url);
  void newLocalSocketConnection();
    
  void startTor(); /** start tor */ 
  void startFailed(QString errmsg); /** Called when the Tor process fails to start. */  
  void started();/** Called when the Tor process has successfully started. */  
  void stopped(int errorCode, QProcess::ExitStatus exitStatus);/** Called when the Tor process has exited, either expectedly or not. */
  void connected(); /** Called when the control socket has connected to Tor. */
  void connectFailed(QString errmsg); /** Called when the control connection fails. */
  void disconnect(); /** Called when TorNavigator wants to disconnect from a Tor it did not start. */
  void disconnected(); /** Called when the control socket has been disconnected. */  
  bool authenticate();/** Authenticates TorNavigator to Tor's control port. */
  void authenticated();/** Called when TorNavigator has successfully authenticated to Tor. */
  void authenticationFailed(QString errmsg); /** Called when TorNavigator fails to authenticate to Tor. The failure reason is
   * specified in <b>errmsg</b>. */   
//  bool stop();/** Called when the user selects "Stop" form the menu. */
  /** Called when the application has started and the main event loop is
   * running. */
  void mainLoopRunning();
  /** Terminate the Tor process if it is being run under TorNavigator, disconnect
   * all TorControl signals, and exit TorNavigator. */
  void aboutToQuit();

private:

  enum TorStatus {
    Unset,                      /**< Tor's status has not yet been set. */
    Stopping,                   /**< Tor is in the process of shutting down. */
    Stopped,                    /**< Tor is not running. */
    Starting,                   /**< Tor is in the process of starting. */
    Started,                    /**< Tor is currently running. */
    Authenticating,             /**< TorNavigator is authenticating to Tor. */
    Authenticated,              /**< TorNavigator has authenticated to Tor. */
    CircuitEstablished          /**< Tor has built a circuit. */
  };
  TorStatus _status;              /** The current status of Tor. */ 
  bool _isIntentionalExit;        /** Used to determine if the Tor process exiting was intentional or not */ 
  bool _delayedShutdownStarted;   /** Tracks whether we started a delayed server shutdown. */ 
  bool _isTorNavigatorRunningTor; /** Set to true if TorNavigator started its own Tor process. */ 
  
  
    /** Catches debugging messages from Qt and sends them to 
   * TorNavigator's logs. */
   static void qt_msg_handler(QtMsgType type, const char *msg);   
   /** Called when Tor thinks the user has tried to connect to a port that
   * typically is used for unencrypted applications. Warns the user and allows
   * them to ignore future warnings on <b>port</b>. */
   void warnDangerousPort(quint16 port, bool rejected);
    TorStatus updateTorStatus(TorStatus status);  /** Updates the UI to reflect Tor's current <b>status</b>. Returns the
   * previously set TorStatus value. */   
   void setStartupProgress(int percentComplete, const QString &description);/** Sets the progress bar completion value to <b>progressValue</b> and sets
   * the status text to <b>description</b>. */
   void setStartupProgressVisible(bool visible);/** Sets the visibility of the startup status description and progress bar
   * to <b>visible</b>. */
   /** Called when Tor has successfully established a circuit. */
   void circuitEstablished();
   /** Searches for and attempts to load the control authentication cookie.
   * This assumes the cookie is named 'control_auth_cookie'. If
   * <b>cookiePath</b> is empty, this method will search some default locations
   * depending on the current platform. <b>cookiePath</b> can point to either
   * a cookie file or a directory containing the cookie file. */
   QByteArray loadControlCookie(QString cookiePath = QString());
   /** Set to true if we should use the control password saved in TorSettings
   * when authenticating to Tor. */
   bool _useSavedPassword;
   /** Remembers the control password between when we start Tor with a hash of
   * the password and when we need to provide the password itself. */
   QString _controlPassword;   
   void checkTorVersion();
   /** Called when Tor thinks its version is old or unrecommended, and displays
   * a message notifying the user. */
   void dangerousTorVersion();
   /** Called when Tor's bootstrapping status changes. <b>bse</b> represents
   * Tor's current estimate of its bootstrapping progress. */
   void bootstrapStatusChanged(const BootstrapStatus &bs);   
      
    void clean();
    void installTranslator(const QString &name);

    static HistoryManager *s_historyManager;
    static DownloadManager *s_downloadManager;
    static NetworkAccessManager *s_networkAccessManager;
    static BookmarksManager *s_bookmarksManager;
    
    QList<QPointer<BrowserMainWindow> > m_mainWindows;
    QLocalServer *m_localServer;
    QByteArray m_lastSession;
    mutable QIcon m_defaultIcon;
    
   /** Clears the list of default CA certificates and adds only the ones
   * TorNavigator is interested in.
   */
   void loadDefaultCaCertificates() const;
      
    /**TorControl *_torControl */
    PrivoxyController m_PrivoxyController;   
    void updatePrivoxyConfig();
    
   static TorControl* _torControl;      /**< TorNavigator's main TorControl object.*/
   static Log _log;                     /**< Logs debugging messages to file or stdout. */   
};



#endif // BROWSERAPPLICATION_H

