/****************************************************************************
** TOR NAVIGATOR - PRIVOXY CONTROLLER
** 
** (C) Cinarchesi
** 
**GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
****************************************************************************/ 


#include <QProcess>
#include <QMessageBox>
#include <QFile>
#include <QTextStream>

#include "browsermainwindow.h"
#include "browserapplication.h"
#include "privoxycontroller.h"

#if defined (Q_OS_WIN32)
#include <windows.h>
#endif

//
PrivoxyController::PrivoxyController(  ) 
	: QObject()
{
     status="not running";
}
//
PrivoxyController::~PrivoxyController()
{
}

// start privoxy service
void PrivoxyController::startPrivoxy()
{
    QString program;
    QString configfile;	
    
    program=QApplication::applicationDirPath()+"/privoxy/linux/privoxy";
    configfile=QApplication::applicationDirPath()+"/privoxy/linux/config";
	
     #if defined (Q_OS_WIN32)
     program =QApplication::applicationDirPath()+"/privoxy/win32/privoxy.exe";
     configfile=QApplication::applicationDirPath()+"/privoxy/win32/config.txt";
     #endif
         
     QStringList args;    
     
     args <<"--no-daemon";    
     args <<configfile;
     

     privoxyProcess = new QProcess();
     privoxyProcess->setReadChannelMode(QProcess::MergedChannels);
     privoxyProcess->start(program, args);
     
     privoxyProcess->waitForStarted();
     status="running";
     
     
     /*Q_PID*/quint64 privoxyPid= getPrivoxyPid();
     QString privoxyPidStr = tr("Pid = ") + QString::number(privoxyPid,10);    
     
BrowserMainWindow *mainWindow = BrowserApplication::instance()->mainWindow();
mainWindow->m_torDialog->privoxyLogAddMsg(tr("Privoxy started...")+ privoxyPidStr);
mainWindow->m_privoxyAction->setEnabled(true);


connect(privoxyProcess, SIGNAL(started ()),
            this, SLOT(slotPrivoxyStarted())); 
    	  
connect(privoxyProcess, SIGNAL(error(QProcess::ProcessError)),
            this,SLOT(slotPrivoxyError( QProcess::ProcessError)));
              
connect(privoxyProcess, SIGNAL(readyReadStandardOutput()),
            this,SLOT(slotReadPrivoxyStandartOutput()));
            
connect(privoxyProcess, SIGNAL(finished (int, QProcess::ExitStatus)),
            this,SLOT(slotPrivoxyFinished(int,QProcess::ExitStatus))); 
}


QString PrivoxyController::writePrivoxyConf()
{
	// TODO  ? autogenerated privoxy config file 
/**
# Generally, this file goes in /etc/privoxy/config
#
# Tor listens as a SOCKS4a proxy here:
forward-socks4a / 127.0.0.1:9050 .
confdir /etc/privoxy
logdir /var/log/privoxy
actionsfile standard  # Internal purpose, recommended
actionsfile default   # Main actions file
actionsfile user      # User customizations
filterfile default.filter

# Don't log interesting things, only startup messages, warnings and errors
#logfile logfile
#jarfile jarfile
#debug   0    # show each GET/POST/CONNECT request
debug   4096 # Startup banner and warnings
debug   8192 # Errors - *we highly recommended enabling this*

user-manual /usr/share/doc/privoxy/user-manual
listen-address  127.0.0.1:8118
toggle  1
enable-remote-toggle 0
enable-edit-actions 0
enable-remote-http-toggle 0
buffer-limit 4096
*/ 
 
	QFile tfPrivoxy("privoxyconfig");	
    return tfPrivoxy.fileName();
}

void PrivoxyController::slotPrivoxyStarted()
{
BrowserMainWindow *mainWindow = BrowserApplication::instance()->mainWindow();  
mainWindow->m_privoxyAction->setEnabled(true);
status="running";	
}


void PrivoxyController::slotPrivoxyFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
BrowserMainWindow *mainWindow = BrowserApplication::instance()->mainWindow();
if (exitStatus==QProcess::NormalExit) mainWindow->m_torDialog->privoxyLogAddMsg((tr("Exit code %1 :Privoxy process exited normally.").arg(exitCode)));
if (exitStatus==QProcess::CrashExit)  mainWindow->m_torDialog->privoxyLogAddMsg(tr("Exit code %1 : Privoxy process crashed.").arg(exitCode));
mainWindow->m_privoxyAction->setEnabled(false);
status="stopped"; 	
}


void PrivoxyController::slotReadPrivoxyStandartOutput()
{
QByteArray bytearray;
bytearray=privoxyProcess->readAll();
QString str(bytearray);

BrowserMainWindow *mainWindow = BrowserApplication::instance()->mainWindow();
mainWindow->m_torDialog->privoxyLogAddMsg(str);  
}

void PrivoxyController::slotPrivoxyError(QProcess::ProcessError error)
{	
QString errmsg;
   switch (error)
   {
case 0:
	 errmsg= tr("The process failed to start. Either the invoked program is missing, or you may have insufficient permissions to invoke the program.");	
     break;
case 1:
	 errmsg= tr("The process crashed some time after starting successfully.");
     break;
case 2:
	 errmsg= tr("The last waitFor...() function timed out. The state of QProcess is unchanged, and you can try calling waitFor...() again.");
     break;
case 3:
	 errmsg= tr("An error occurred when attempting to write to the process. For example, the process may not be running, or it may have closed its input channel.");	 	
     break;
case 4:
 	 errmsg= tr("An error occurred when attempting to read from the process. For example, the process may not be running.");
     break;
case 5:
	 errmsg= tr("An unknown error occurred. This is the default return value of error().");
	 break;
   }
	 
BrowserMainWindow *mainWindow = BrowserApplication::instance()->mainWindow();
mainWindow->m_torDialog->privoxyLogAddMsg(errmsg); 

}


qint64 PrivoxyController::getPrivoxyPid()
{
#if defined (Q_OS_WIN32)
    return (qint64)((privoxyProcess->pid())->dwProcessId);
#else
    return privoxyProcess->pid();
#endif
}

